/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LUACONTROLLER_H_
#define LUACONTROLLER_H_
extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}
#include "lua/Luna.h"
#include "lua/LuaTable.h"
#include "lua/LuaSearch.h"
#include "controller/Controller.h"
#include <QObject>

/**
 * \class LuaController
 * \brief Controller for the Lua functions.
 *
 * This class initializes and controls the communication with the Lua-C-API.
 * Its Register template allows to register C++-objects in the current
 * lua_state and to use them with the LuaXXX wrapper classes.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 * \see Luna
 */
class LuaController: public Controller
{

Q_OBJECT

public:
    LuaController  ( QObject* );

    int doString ( QString& );
    int doFile   ( QString  );

    void close ();

    void print           ( QString   );
    QString reportErrors ( const int );

    /**
     * \brief Template function for registering C++-objects in Lua.
     *
     * Registers C++-objects in the lua_state.
     *
     * \param x Object to register.
     * \param name The name of the object in Lua.
     * \see Luna::Register()
     */
    template < class T, class S >
    void Register ( S * x, QString name )
    {
        Luna<T>::Register ( L_ );
        lua_pushlightuserdata ( L_, (void*) x );
        lua_setglobal ( L_,name.toLatin1().data() );
    }



private:
    /**
     * \brief Reference on the lua_State, that is used for all calls to Lua.
     */
    lua_State* L_;

    void init ();

    /**
     * \brief Is true while a string or file is executing.
     */
    bool executing;

    /**
     * \brief Static variable to stop the current execution.
     *
     * \see lstop()
     */
    static bool quit;

    static void lstop ( lua_State*, lua_Debug* );

protected slots:
    void getCompletionList ( const QString&, QStringList& );
    void stop ();

signals:
    /**
     * \brief Print signal.
     *
     * \param text Text to print.
     */
    void printSignal ( QString text );

    /**
     * \brief Execution starts.
     *
     */
    void startExecution();
    /**
     * \brief Execution ends.
     */
    void endExecution();
    /**
     * \brief An error while execution.
     *
     * Sends the error message with the signal.
     *
     * \param error Error Message.
     */
    void error(QString error);
};

#endif /* LUACONTROLLER_H_ */
