/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SEARCHCONTROLLER_H_
#define SEARCHCONTROLLER_H_
#include <QObject>
#include "gui/UCQTableView.h"
#include "data/TableModel.h"
#include "controller/Controller.h"

/**
 * \class SearchController
 * \brief Controller for the search functionality, that manages and performs
 *        search operations.
 *
 * The SearchController provides methods to search in the data of a
 * TableModel object. The search can be performed on the visible table
 * (search()) or the whole dataset of the model (searchTable()). It is possible
 * to search for exact matches or imprecise matches and regular expressions.
 * The parameters of the search methods allow to specify, which datafields of
 * the letters should be included into the search (codepoint, collations,
 * comment). The hits of a search are stored in an intern list. They can be
 * visualized by selecting them in the TableView with the next(), previous()
 * and selectAll() methods. It is also possible, to append them to the currently
 * visible table with showItems(). Classes that utilize the SearchController
 * are UCQSearchDialog and LuaSearch. For regular expressions the QRegExp class
 * is used.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 * \see TableModel
 * \see UCQTableView
 * \see UCQSearchDialog
 * \see LuaSearch
 */
class SearchController: public Controller
{

Q_OBJECT

public:
    SearchController  ( QObject*, TableModel*, UCQTableView* );

    bool search      ( QString, bool, bool, bool, bool, bool, bool, QList<bool>& );
    void searchTable ( QString, bool, bool, bool, bool, bool, bool, QList<bool>& );

    void next        ();
    void previous    ();
    void selectAll   ();
    void clear       ();
    void addItems    ();
    int numberOfHits ();

    bool hasNext     ();
    bool hasPrevious ();
    bool isEmpty     ();

    QList<bool> getLvls     () const;
    bool getCasesensitivity () const;
    bool getCodepoint       () const;
    bool getCollation       () const;
    bool getComment         () const;
    bool getRegexp          () const;
    QString getSearchtext   () const;

    void setCasesensitivity ( bool );
    void setCodepoint       ( bool );
    void setCollation       ( bool );
    void setComment         ( bool );
    void setRegexp          ( bool );
    void setSearchtext      ( QString     );
    void setLvls            ( QList<bool> );

    void setAllBlocks		();
private:
    QList<Letter*> searchForString ();
    QList<Letter*> searchForRegExp ();
    /**
     * \brief The model that is used for search operations.
     */
    TableModel * model_;
    /**
     * \brief The view in which results are shown.
     */
    UCQTableView * view_;
    /**
     * \brief Index for the findlist_.
     */
    int findindex_;
    /**
     * \brief List of hits.
     */
    QList<Letter *> findlist_;
    /**
     * \brief List of Letters to search in.
     */
    QList<Letter*> letters_;
    /**
     * \brief If true, comments are included in the search.
     */
    bool comment_;
    /**
     * \brief If true, collations are included in the search.
     */
    bool collation_;
    /**
     * \brief If true, codepoints are included in the search.
     */
    bool codepoint_;
    /**
     * \brief If true, linestring are included in the search.
     */
    bool linestring_;
    /**
     * \brief If true, the searchtext is regarded as a
     *        QRegExp and allows regular expressions and imprecise search.
     */
    bool regexp_;
    /**
     * \brief If true, the comment search is case sensitive.
     */
    bool casesensitivity_;
    /**
     * \brief Contains four bool values that determine which levels are
     *        included in a search containing collations.
     */
    QList<bool> lvls_;
    /**
     * \brief The text/regex to search for.
     */
    QString searchtext_;

signals:
    /**
     * \brief Signal to select letters.
     *
     * \param index The index of the Letter to select.
     * \param selected If true, the Letter is selected.
     */
    void selectLetter ( QModelIndex index, bool selected );

    /**
     * \brief Signal to select a list of letters.
     *
     * \param indices List of indices of Letters to select.
     * \param selected If true, the Letters are selected.
     */
    void selectLetters ( const QModelIndexList& indices, bool selected );

    /**
     * \brief Signal to scroll to a letter in the table view.
     *
     * \param index Index of a Letter in the View.
     */
    void scrollTo ( QModelIndex index );

    /**
     * \brief Signal to clear the current selection.
     */
    void clearSelection ();

    /**
     * \brief Signal to add letters to the tableview.
     *
     * \param letters QList with pointers on Letters that are to be added to
     *                the visible list of the TableModel.
     */
    void addLetters ( QList<Letter*> letters);
};

#endif /* SEARCHCONTROLLER_H_ */
